@@ -23,7 +23,8 @@
 
 #include "dbus-values.h"
 
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/compat.h>
 
 #include <gjs-dbus/dbus.h>
 #include <util/log.h>
@@ -51,7 +52,7 @@ gjs_js_one_value_from_dbus(JSContext       *context,
             DBusMessageIter struct_iter;
             int index;
 
-            obj = JS_NewArrayObject(context, 0, JSVAL_NULL);
+            obj = JS_NewArrayObject(context, 0, NULL);
             if (obj == NULL)
                 return JS_FALSE;
 
@@ -61,20 +62,20 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                 jsval prop_value;
 
                 prop_value = JSVAL_VOID;
-                JS_AddRoot(context, &prop_value);
+                JS_AddValueRoot(context, &prop_value);
                 if (!gjs_js_one_value_from_dbus(context, &struct_iter, &prop_value)) {
-                    JS_RemoveRoot(context, &prop_value);
+                    JS_RemoveValueRoot(context, &prop_value);
                     return JS_FALSE;
                 }
 
                 if (!JS_DefineElement(context, obj,
                                       index, prop_value,
                                       NULL, NULL, JSPROP_ENUMERATE)) {
-                    JS_RemoveRoot(context, &prop_value);
+                    JS_RemoveValueRoot(context, &prop_value);
                     return JS_FALSE;
                 }
 
-                JS_RemoveRoot(context, &prop_value);
+                JS_RemoveValueRoot(context, &prop_value);
                 dbus_message_iter_next(&struct_iter);
                 index ++;
             }
@@ -94,49 +95,66 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                 if (obj == NULL)
                     return JS_FALSE;
 
-                JS_AddRoot(context, &obj);
+                JS_AddObjectRoot(context, &obj);
                 dbus_message_iter_recurse(iter, &array_iter);
                 while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
                     DBusMessageIter entry_iter;
+                    jsval key_value, entry_value;
+                    JSString *key_str;
                     const char *key;
-                    jsval entry_value;
 
                     dbus_message_iter_recurse(&array_iter, &entry_iter);
 
-                    if (dbus_message_iter_get_arg_type(&entry_iter) != DBUS_TYPE_STRING) {
-                        gjs_throw(context, "Dictionary keys are not strings, can't convert to JavaScript");
-                        JS_RemoveRoot(context, &obj);
+                    if (!dbus_type_is_basic(dbus_message_iter_get_arg_type(&entry_iter))) {
+                        gjs_throw(context, "Dictionary keys are not a basic type, can't convert to JavaScript");
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
-                    dbus_message_iter_get_basic(&entry_iter, &key);
+                    entry_value = JSVAL_VOID;
+                    JS_AddValueRoot(context, &entry_value);
+                    if (!gjs_js_one_value_from_dbus(context, &entry_iter, &key_value)) {
+                        JS_RemoveValueRoot(context, &key_value);
+                        JS_RemoveObjectRoot(context, &obj);
+                        return JS_FALSE;
+                    }
+
+                    key_str = JS_ValueToString(context, key_value);
+                    JS_AddStringRoot(context, &key_str);
+                    key = JS_GetStringBytes(key_str);
 
                     dbus_message_iter_next(&entry_iter);
 
                     gjs_debug_dbus_marshal("Defining dict entry %s in jsval dict", key);
 
                     entry_value = JSVAL_VOID;
-                    JS_AddRoot(context, &entry_value);
+                    JS_AddValueRoot(context, &entry_value);
                     if (!gjs_js_one_value_from_dbus(context, &entry_iter, &entry_value)) {
-                        JS_RemoveRoot(context, &entry_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &key_value);
+                        JS_RemoveStringRoot(context, &key_str);
+                        JS_RemoveValueRoot(context, &entry_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
                     if (!JS_DefineProperty(context, obj,
                                            key, entry_value,
                                            NULL, NULL, JSPROP_ENUMERATE)) {
-                        JS_RemoveRoot(context, &entry_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &key_value);
+                        JS_RemoveStringRoot(context, &key_str);
+                        JS_RemoveValueRoot(context, &entry_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
-                    JS_RemoveRoot(context, &entry_value);
+                    JS_RemoveValueRoot(context, &key_value);
+                    JS_RemoveStringRoot(context, &key_str);
+                    JS_RemoveValueRoot(context, &entry_value);
                     dbus_message_iter_next(&array_iter);
                 }
 
                 *value_p = OBJECT_TO_JSVAL(obj);
-                JS_RemoveRoot(context, &obj);
+                JS_RemoveObjectRoot(context, &obj);
             } else if (elem_type == DBUS_TYPE_BYTE) {
                 /* byte arrays go to a string */
                 const char *v_BYTES;
@@ -154,38 +172,38 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                 DBusMessageIter array_iter;
                 int index;
 
-                obj = JS_NewArrayObject(context, 0, JSVAL_NULL);
+                obj = JS_NewArrayObject(context, 0, NULL);
                 if (obj == NULL)
                     return JS_FALSE;
 
-                JS_AddRoot(context, &obj);
+                JS_AddObjectRoot(context, &obj);
                 dbus_message_iter_recurse(iter, &array_iter);
                 index = 0;
                 while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
                     jsval prop_value;
 
                     prop_value = JSVAL_VOID;
-                    JS_AddRoot(context, &prop_value);
+                    JS_AddValueRoot(context, &prop_value);
                     if (!gjs_js_one_value_from_dbus(context, &array_iter, &prop_value)) {
-                        JS_RemoveRoot(context, &prop_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &prop_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
                     if (!JS_DefineElement(context, obj,
                                           index, prop_value,
                                           NULL, NULL, JSPROP_ENUMERATE)) {
-                        JS_RemoveRoot(context, &prop_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &prop_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
-                    JS_RemoveRoot(context, &prop_value);
+                    JS_RemoveValueRoot(context, &prop_value);
                     dbus_message_iter_next(&array_iter);
                     index ++;
                 }
                 *value_p = OBJECT_TO_JSVAL(obj);
-                JS_RemoveRoot(context, &obj);
+                JS_RemoveObjectRoot(context, &obj);
             }
         }
         break;
@@ -240,7 +258,7 @@ gjs_js_one_value_from_dbus(JSContext       *context,
         {
             double v_DOUBLE;
             dbus_message_iter_get_basic(iter, &v_DOUBLE);
-            if (!JS_NewDoubleValue(context, v_DOUBLE, value_p))
+            if (!JS_NewNumberValue(context, v_DOUBLE, value_p))
                 return JS_FALSE;
         }
         break;
@@ -291,25 +309,27 @@ gjs_js_values_from_dbus(JSContext          *context,
     jsval value;
 
     value = JSVAL_VOID;
-    JS_AddRoot(context, &value);
+    JS_AddValueRoot(context, &value);
 
     *array_p = NULL;
 
     array = gjs_rooted_array_new();
 
-    do {
-        if (!gjs_js_one_value_from_dbus(context, iter, &value)) {
-            gjs_rooted_array_free(context, array, TRUE);
-            JS_RemoveRoot(context, &value);
-            return JS_FALSE; /* error message already set */
-        }
+    if (dbus_message_iter_get_arg_type(iter) != DBUS_TYPE_INVALID) {
+        do {
+            if (!gjs_js_one_value_from_dbus(context, iter, &value)) {
+                gjs_rooted_array_free(context, array, TRUE);
+                JS_RemoveValueRoot(context, &value);
+                return JS_FALSE; /* error message already set */
+            }
 
-        gjs_rooted_array_append(context, array, value);
-    } while (dbus_message_iter_next(iter));
+            gjs_rooted_array_append(context, array, value);
+        } while (dbus_message_iter_next(iter));
+    }
 
     *array_p = array;
 
-    JS_RemoveRoot(context, &value);
+    JS_RemoveValueRoot(context, &value);
 
     return JS_TRUE;
 }
@@ -585,12 +605,12 @@ append_array(JSContext         *context,
         dbus_signature_iter_recurse(sig_iter, &element_sig_iter);
 
         dbus_message_iter_open_container(iter, DBUS_TYPE_STRUCT, NULL, &struct_iter);
-        
+
         have_next = dbus_signature_iter_get_current_type(&element_sig_iter) != DBUS_TYPE_INVALID;
 
         for (i = 0; i < length; i++) {
             element = JSVAL_VOID;
-            
+
             if (!have_next) {
                 gjs_throw(context, "Insufficient elements for structure in JS Array");
                 return JS_FALSE;
@@ -606,10 +626,10 @@ append_array(JSContext         *context,
             if (!gjs_js_one_value_to_dbus(context, element, &struct_iter,
                                           &element_sig_iter))
                 return JS_FALSE;
-            
+
             have_next = dbus_signature_iter_next (&element_sig_iter);
         }
-        
+
         if (have_next) {
             gjs_throw(context, "Too many elements for structure in JS Array");
             return JS_FALSE;
@@ -644,7 +664,7 @@ append_array(JSContext         *context,
         }
 
         dbus_message_iter_close_container(iter, &array_iter);
-        
+
         return JS_TRUE;
     } else {
       gjs_throw(context,
@@ -756,11 +776,11 @@ append_dict(JSContext         *context,
         return JS_FALSE;
     }
 
-    prop_id = JSVAL_VOID;
+    prop_id = JSID_VOID;
     if (!JS_NextProperty(context, props_iter, &prop_id))
         return JS_FALSE;
 
-    while (prop_id != JSVAL_VOID) {
+    while (!JSID_IS_VOID(prop_id)) {
         jsval nameval;
         char *name;
         jsval propval;
@@ -844,7 +864,7 @@ append_dict(JSContext         *context,
         dbus_message_iter_close_container(&dict_iter, &entry_iter);
 
     next:
-        prop_id = JSVAL_VOID;
+        prop_id = JSID_VOID;
         if (!JS_NextProperty(context, props_iter, &prop_id))
             return JS_FALSE;
     }
@@ -1011,26 +1031,3 @@ gjs_js_values_to_dbus(JSContext         *context,
     return JS_TRUE;
 }
 
-/* If jsval is an object, add properties from the DBusMessage such as the
- * sender. If jsval is not an object, do nothing.
- */
-JSBool
-gjs_js_add_dbus_props(JSContext       *context,
-                      DBusMessage     *message,
-                      jsval            value)
-{
-    const char *sender;
-
-    if (!JSVAL_IS_OBJECT(value))
-        return JS_TRUE;
-
-    sender = dbus_message_get_sender(message);
-
-    if (!JS_DefineProperty(context, JSVAL_TO_OBJECT(value),
-                           "_dbus_sender",
-                           STRING_TO_JSVAL(JS_NewStringCopyZ(context, sender)),
-                           NULL, NULL, JSPROP_ENUMERATE))
-        return JS_FALSE;
-
-    return JS_TRUE;
-}
